/* --- language.js ----- */

define(['jquery', 'app'], function($, app) {
	
	var lang;
	var defLang;
	var supportedLanguages;
	var languages;
	var filePath;
	var translations = {};

	var mod = {
		'init':           init,
		'set':            setLanguage,
		'get':            getLanguage,
		'setPreferred':   setPreferredLanguage,
		'isSupported':    isLanguageSupported,
		'isSet':          isLanguageSet,
		'getTranslation': getTranslation
	};

	/**
	 * init
	 *
	 * Initialize module.
	 * @param   {Object} config            Config object
	 * @param   {String} cofig.default     Default language
	 * @param   {Object} config.supported  Supported languages: {'locale': 'abbr'}
	 * @returns {Object}                   Module interface
	 */
	function init(config) {
		defLang = config.default;
		supportedLanguages = config.supported;
		languages = $.map(supportedLanguages, function(lang) {
			return lang;
		});
		filePath = config.filePath;

		return mod;
	}

	/**
	 * setLanguage
	 *
	 * Set the app language, store it in localStorage and get its translations.
	 * @param   {String} newLang  Language to set, e.g. 'nl'
	 */
	function setLanguage(newLang) {
		if (newLang !== lang && mod.isSupported(newLang)) {
			//app.trans = mod.getTranslation(newLang);
			app.localStorage.setItem('lang', newLang);
			lang = newLang;
			$(document).triggerHandler('onsetlanguage', [newLang]);
		}
	}

	/**
	 * getLanguage
	 *
	 * Get the language from localStorage.
	 * @returns {String}  Stored language, e.g. 'nl'
	 */
	function getLanguage() {
		return app.localStorage.getItem('lang');
	}

	/**
	 * setPreferredLanguage
	 *
	 * Set the preferred language based on navigator settings and switch to that language.
	 */
	function setPreferredLanguage() {
		if (navigator.globalization && navigator.globalization.getPreferredLanguage) {
			navigator.globalization.getPreferredLanguage(
				function(lang) {
					mod.set(mod.isSupported(lang) ? lang : defLang);
				},
				function(error) {
					mod.set(defLang);
				}
			);
		} else {
			mod.set(defLang);
		}
	}

	/**
	 * isLanguageSupported
	 * 
	 * Check whether the given language is supported.
	 * @param   {String}  lang  The language to check, e.g. 'nl'
	 * @returns {Boolean}
	 */
	function isLanguageSupported(lang) {
		return !!(lang && supportedLanguages[lang]) || $.inArray(lang, languages) != -1;
	}

	/**
	 * isLanguageSet
	 * 
	 * Check whether a language has been set.
	 * @returns {Boolean}
	 */ 
	function isLanguageSet() {
		return !!mod.get();
	}

	/**
	 * getTranslation
	 *
	 * Get translation for a given language.
	 * @param   {String} [lang]  Language to get the translation for. Default: language set for app.
	 * @returns {Object}         Translation
	 */
	function getTranslation(lang) {
		if (!mod.isSupported(lang)) {
			return;
		}

		if (translations[lang]) {
			return translations[lang];
		}

		translations[lang] = {};
		require([filePath + lang + '.json?callback=define'], function(translation) {
			$.extend(true, translations[lang], translation || {});
		});

		return translations[lang];
	}

	return {
		'init': init
	};
});